@@ -1,6 +1,7 @@
 /*
- This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2010.
- Authors: Andreas Knuepfer, Holger Brunst, Ronny Brendel, Thomas Kriebitzsch
+ This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2011.
+ Authors: Andreas Knuepfer, Holger Brunst, Ronny Brendel, Thomas Kriebitzsch 
+ also: patches by Rainer Keller, thanks a lot!
 */
 
 #ifdef HAVE_CONFIG_H
@@ -19,6 +20,7 @@
 #include "OTF_Platform.h"
 #include "OTF_Errno.h"
 
+#include "OTF_Keywords.h"
 
 /** constructor - internal use only */
 int OTF_RBuffer_init( OTF_RBuffer* rbuffer );
@@ -161,6 +163,8 @@ int OTF_RBuffer_init( OTF_RBuffer* rbuffer ) {
 	rbuffer->firstTime= (uint64_t) -1;
 	rbuffer->lastTime= (uint64_t) -1;
 	
+	rbuffer->list = NULL;
+	
 #ifdef HAVE_ZLIB
 	rbuffer->zbuffersize= 1024*10;
 #endif /* HAVE_ZLIB */
@@ -186,38 +190,36 @@ int OTF_RBuffer_finish( OTF_RBuffer* rbuffer ) {
 
 OTF_RBuffer* OTF_RBuffer_open( const char* filename, OTF_FileManager* manager ) {
 
+	OTF_RBuffer* ret;
 
-	OTF_RBuffer* ret= (OTF_RBuffer*) malloc( sizeof(OTF_RBuffer) );
+	/* Check the input parameters */
+	if( NULL == manager ) {
+		
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
+				"manager has not been defined.\n",
+				__FUNCTION__, __FILE__, __LINE__ );
+
+		return NULL;
+	}
+
+	ret= (OTF_RBuffer*) malloc( sizeof(OTF_RBuffer) );
 	if ( NULL == ret ) {
 
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"no memory left.\n",
 				__FUNCTION__, __FILE__, __LINE__ );
 				
 		return NULL;
 	}
 
-
 	OTF_RBuffer_init( ret );
 
-	if( NULL == manager ) {
-		
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
-				"manager has not been defined.\n",
-				__FUNCTION__, __FILE__, __LINE__ );
-
-		free( ret );
-		ret= NULL;
-				
-		return NULL;
-	}
-	
 	ret->file= OTF_File_open( filename, manager, OTF_FILEMODE_READ );
 
 	if ( NULL == ret->file ) {
 
 /* *** commented because it can happen when defstream cannot be loaded
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"OTF_File_open() failed. filename '%s'\n",
 				__FUNCTION__, __FILE__, __LINE__, filename );
 */
@@ -228,6 +230,73 @@ OTF_RBuffer* OTF_RBuffer_open( const char* filename, OTF_FileManager* manager )
 		return ret;
 	}
 
+	ret->list = OTF_KeyValueList_new();
+
+	if (ret->list == NULL) {
+
+		OTF_File_close(ret->file);
+		ret->file = NULL;
+
+		free( ret );
+		ret= NULL;
+				
+		return NULL;
+	}
+
+#ifdef HAVE_ZLIB
+	OTF_File_setZBufferSize( ret->file, ret->zbuffersize );
+#endif /* HAVE_ZLIB */
+
+	return ret;
+}
+
+
+OTF_RBuffer* OTF_RBuffer_open_with_external_buffer( uint32_t len, const char* buffer, uint8_t is_compressed ) {
+
+
+	OTF_RBuffer* ret;
+
+
+	ret= (OTF_RBuffer*) malloc( sizeof(OTF_RBuffer) );
+	if ( NULL == ret ) {
+
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
+				"no memory left.\n",
+				__FUNCTION__, __FILE__, __LINE__ );
+
+		return NULL;
+	}
+
+	OTF_RBuffer_init( ret );
+
+	ret->file= OTF_File_open_with_external_buffer( len, buffer, is_compressed, OTF_FILEMODE_READ );
+	if ( NULL == ret->file ) {
+
+/* *** commented because it can happen when defstream cannot be loaded
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
+				"OTF_File_open() failed. filename '%s'\n",
+				__FUNCTION__, __FILE__, __LINE__, filename );
+*/
+
+		free( ret );
+		ret= NULL;
+
+		return ret;
+	}
+
+	ret->list = OTF_KeyValueList_new();
+
+	if (ret->list == NULL) {
+
+		OTF_File_close(ret->file);
+		ret->file = NULL;
+
+		free( ret );
+		ret= NULL;
+				
+		return NULL;
+	}
+
 #ifdef HAVE_ZLIB
 	OTF_File_setZBufferSize( ret->file, ret->zbuffersize );
 #endif /* HAVE_ZLIB */
@@ -245,6 +314,10 @@ int OTF_RBuffer_close( OTF_RBuffer* rbuffer ) {
 	
 	rbuffer->file= NULL;
 
+	if( rbuffer->list != NULL) {
+		OTF_KeyValueList_close( rbuffer->list);
+	}
+
 	OTF_RBuffer_finish( rbuffer );
 
 	free( rbuffer );
@@ -253,13 +326,12 @@ int OTF_RBuffer_close( OTF_RBuffer* rbuffer ) {
 	return ret;
 }
 
-
 int OTF_RBuffer_setSize( OTF_RBuffer* rbuffer, size_t size ) {
 
 
 	if ( size < 100 ) {
 	
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"buffer size %u too small, rejected.\n",
 				__FUNCTION__, __FILE__, __LINE__, (uint32_t) size );
 
@@ -268,7 +340,7 @@ int OTF_RBuffer_setSize( OTF_RBuffer* rbuffer, size_t size ) {
 
 	if ( size < rbuffer->size ) {
 
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"cannot shrink buffer from %u to %u.\n",
 				__FUNCTION__, __FILE__, __LINE__, (uint32_t) rbuffer->size,
 				(uint32_t) size );
@@ -281,7 +353,7 @@ int OTF_RBuffer_setSize( OTF_RBuffer* rbuffer, size_t size ) {
 		rbuffer->size * sizeof(char) );
 	if( NULL == rbuffer->buffer ) {
 		
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"no memory left.\n",
 				__FUNCTION__, __FILE__, __LINE__ );
 
@@ -304,7 +376,7 @@ void OTF_RBuffer_setZBufferSize( OTF_RBuffer* rbuffer, uint32_t size ) {
 	
 	if ( 32 > size ) {
 	
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"intended zbuffer size %u is too small, rejected.\n",
 				__FUNCTION__, __FILE__, __LINE__, size );
 		
@@ -312,13 +384,13 @@ void OTF_RBuffer_setZBufferSize( OTF_RBuffer* rbuffer, uint32_t size ) {
 
 	} else if ( 512 > size ) {
 	
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Warning( "WARNING in function %s, file: %s, line: %i:\n "
 				"zbuffer size %u is very small, accepted though.\n",
 				__FUNCTION__, __FILE__, __LINE__, size );
 
 	} else if ( 10 * 1024 *1024 < size ) {
 
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Warning( "WARNING in function %s, file: %s, line: %i:\n "
 				"zbuffer size %u is rather big, accepted though.\n",
 				__FUNCTION__, __FILE__, __LINE__, size );
 	}
@@ -338,7 +410,7 @@ int OTF_RBuffer_setJumpSize( OTF_RBuffer* rbuffer, size_t size ) {
 
 	if ( size < 100 ) {
 	
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"jump buffer size %u too small, rejected.\n",
 				__FUNCTION__, __FILE__, __LINE__, (uint32_t) size );
 
@@ -347,7 +419,7 @@ int OTF_RBuffer_setJumpSize( OTF_RBuffer* rbuffer, size_t size ) {
 
 	if ( size > rbuffer->size ) {
 
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"buffer size is smaller than jump size %u.\n",
 				__FUNCTION__, __FILE__, __LINE__, (uint32_t) size );
 
@@ -417,7 +489,7 @@ int OTF_RBuffer_guaranteeRecord( OTF_RBuffer* rbuffer ) {
 		/* no complete record available! end of file. */
 
 		/*
-		OTF_fprintf( stderr, "OTF_RBuffer_guaranteeRecord() "
+		OTF_Error( "OTF_RBuffer_guaranteeRecord() "
 			"cannot read, file exceeded\n" );
 		*/
 
@@ -439,16 +511,11 @@ int OTF_RBuffer_guaranteeRecord( OTF_RBuffer* rbuffer ) {
 	while ( 1024*1024*100 > rbuffer->size ) {
 
 
-		/*
-		OTF_fprintf( stderr, "OTF_RBuffer_guaranteeRecord() double buffer size "
-			"%u -> %u!\n", rbuffer->size, 2* rbuffer->size );
-		*/
-
 		ret= OTF_RBuffer_setSize( rbuffer, 2* rbuffer->size );
 
 		if ( 1 != ret ) {
 
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"cannot double buffer size.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 
@@ -458,7 +525,7 @@ int OTF_RBuffer_guaranteeRecord( OTF_RBuffer* rbuffer ) {
 		add= OTF_RBuffer_advance( rbuffer );
 		if ( 0 == add ) {
 
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"file exceeded.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 
@@ -472,7 +539,7 @@ int OTF_RBuffer_guaranteeRecord( OTF_RBuffer* rbuffer ) {
 		}
 	}
 
-	OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+	OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 			"buffer is too small. Extending buffer has finally failed.\n",
 			__FUNCTION__, __FILE__, __LINE__ );
 
@@ -496,7 +563,7 @@ char *OTF_RBuffer_printRecord( OTF_RBuffer* rbuffer ) {
 			ret= (char*) realloc( ret, size + REALLOCSIZE );
 			if( NULL == ret ) {
 				
-				OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+				OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 						"no memory left.\n",
 						__FUNCTION__, __FILE__, __LINE__ );
 
@@ -525,9 +592,17 @@ size_t OTF_RBuffer_advance( OTF_RBuffer* rbuffer ) {
 	size_t ret;
 
 
+	if ( 0 == rbuffer->size ) {
+
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
+			"buffer size not set!\n",
+			__FUNCTION__, __FILE__, __LINE__ );
+		exit(1);
+	}
+
 	if( rbuffer->pos > rbuffer->end ) {
 		
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 			"current position exceeds the file length.\n",
 			__FUNCTION__, __FILE__, __LINE__ );
 
@@ -580,13 +655,11 @@ int OTF_RBuffer_jump( OTF_RBuffer* rbuffer, uint64_t filepos ) {
 	/* uint64_t currentPos; */
 	uint32_t i;
 
-	/* OTF_fprintf( stderr, "\tjump to %llu\n", (unsigned long long) filepos ); */
-
 	ret= OTF_File_seek( rbuffer->file, filepos );
 
 	if ( 0 != ret ) {
 
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"OTF_File_seek() failed.\n",
 				__FUNCTION__, __FILE__, __LINE__ );
 		
@@ -617,7 +690,7 @@ int OTF_RBuffer_jump( OTF_RBuffer* rbuffer, uint64_t filepos ) {
 	if ( 0 == OTF_RBuffer_guaranteeRecord( rbuffer ) ) {
 
 		/*
-		OTF_fprintf( stderr, "OTF_RBuffer_jump() "
+		OTF_Error( "OTF_RBuffer_jump() "
 				"ERROR: could not read far enough\n" );
 		*/
 
@@ -635,7 +708,7 @@ int OTF_RBuffer_jump( OTF_RBuffer* rbuffer, uint64_t filepos ) {
 	if ( 0 == OTF_RBuffer_guaranteeRecord( rbuffer ) ) {
 
 		/*
-		OTF_fprintf( stderr, "OTF_RBuffer_jump() "
+		OTF_Error( "OTF_RBuffer_jump() "
 				"ERROR: could not read far enough\n" );
 		*/
 
@@ -656,7 +729,7 @@ int OTF_RBuffer_jump( OTF_RBuffer* rbuffer, uint64_t filepos ) {
 		if ( 0 == OTF_RBuffer_guaranteeRecord( rbuffer ) ) {
 
 			/*
-			OTF_fprintf( stderr, "OTF_RBuffer_jump() "
+			OTF_Error( "OTF_RBuffer_jump() "
 					"ERROR: could not read far enough\n" );
 			*/
 			return 0;
@@ -681,7 +754,7 @@ int OTF_RBuffer_jump( OTF_RBuffer* rbuffer, uint64_t filepos ) {
 		if ( 0 == OTF_RBuffer_guaranteeRecord( rbuffer ) ) {
 
 			/*
-			OTF_fprintf( stderr, "OTF_RBuffer_jump() "
+			OTF_Error( "OTF_RBuffer_jump() "
 					"ERROR: could not read far enough\n" );
 			*/
 
@@ -741,7 +814,10 @@ uint64_t OTF_RBuffer_getCurrentTime( OTF_RBuffer* rbuffer ) {
 
 
 	if ( rbuffer->time == (uint64_t) -1 ) {
-		OTF_fprintf( stderr, "Warning: Invalid time\n" );
+
+		OTF_Warning( "WARNING in function %s, file: %s, line: %i:\n "
+				"Invalid time.",
+				__FUNCTION__, __FILE__, __LINE__ );
 	}
 
 	return rbuffer->time;
@@ -944,10 +1020,6 @@ int OTF_RBuffer_getFileProperties( OTF_RBuffer* rbuffer ) {
 	}
 
 
-	/*
-	OTF_fprintf( stderr, "search min/max time\n" );
-	*/
-
 	/* get very first timestamp easily */
 	rbuffer->firstTime= rbuffer->time;
 
@@ -957,7 +1029,7 @@ int OTF_RBuffer_getFileProperties( OTF_RBuffer* rbuffer ) {
 	searchRange= ( SEARCH_RANGE < rbuffer->size ) ? SEARCH_RANGE : rbuffer->size;
 	if( 0 >= searchRange ) {
 		
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"the search range is not allowed to be '0'.\n",
 				__FUNCTION__, __FILE__, __LINE__ );
 
@@ -973,7 +1045,6 @@ int OTF_RBuffer_getFileProperties( OTF_RBuffer* rbuffer ) {
 			( pos > searchRange ) ) {
 
 		pos= pos - searchRange;
-		OTF_fprintf( stderr, "  jumppos: %llu\n", (long long unsigned) pos );
 	}*/
 	while ( ( !OTF_RBuffer_jump( rbuffer, pos ) ) &&
 			( pos > 0 ) ) {
@@ -1022,7 +1093,7 @@ int OTF_RBuffer_searchTime( OTF_RBuffer* rbuffer, uint64_t time ) {
 
 	if ( 0 == OTF_RBuffer_getFileProperties( rbuffer ) ) {
 
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"could not determine file size (%llu) or first/last time"
 				" (%llx/%llx)\n",
 				__FUNCTION__, __FILE__, __LINE__,
@@ -1046,7 +1117,7 @@ int OTF_RBuffer_searchTime( OTF_RBuffer* rbuffer, uint64_t time ) {
 		ret= OTF_RBuffer_jump( rbuffer, posA );
 		if ( 1 != ret ) {
 
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"unsuccessful jump to begin pos= %llu.\n",
 					__FUNCTION__, __FILE__, __LINE__, (unsigned long long) posA );
 			
@@ -1104,7 +1175,7 @@ int OTF_RBuffer_searchTime( OTF_RBuffer* rbuffer, uint64_t time ) {
 	ret= OTF_RBuffer_jump( rbuffer, posA );
 	if ( 1 != ret ) {
 
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"unsuccessful jump to begin pos= %llu.\n",
 				__FUNCTION__, __FILE__, __LINE__, (unsigned long long) posA );
 
@@ -1117,7 +1188,7 @@ int OTF_RBuffer_searchTime( OTF_RBuffer* rbuffer, uint64_t time ) {
 		ret2= OTF_RBuffer_getRecord( rbuffer );
 		if( NULL == ret2 ) {
 			
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"OTF_RBuffer_getRecord() failed.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 
@@ -1244,3 +1315,160 @@ uint64_t OTF_RBuffer_getFilePos( OTF_RBuffer* rbuffer ) {
 
 	return OTF_File_tell( rbuffer->file );
 }
+
+uint32_t OTF_RBuffer_readBytes( OTF_RBuffer* rbuffer, uint8_t *array, uint32_t max_len ) {
+
+	char c;
+	uint32_t k = 0;
+	uint32_t i = 0;
+
+	OTF_RBuffer_skipSpaces( rbuffer );
+
+	while( 1 ) {
+
+		if (k == 2) {
+			i++;
+			k = 0;
+		}
+
+		c= rbuffer->buffer[rbuffer->pos];
+
+		if ( '0' <= c && c <= '9' ) {
+          
+            if ( i >= max_len )         
+                return i + 1;
+
+			array[i] = ( array[i] << 4) | ( c - '0' );
+			++(rbuffer->pos);
+
+		} else if ( 'a' <= c && c <= 'f' ) {
+          
+            if ( i >= max_len )
+                return i + 1;
+
+			array[i] = ( array[i] << 4) | ( c - ( 'a' - 10 ) );
+			++(rbuffer->pos);
+
+		} else {
+          
+			return i;
+		}
+
+		k++;
+	}
+}
+
+uint32_t OTF_RBuffer_readKeyValueList(OTF_RBuffer* buffer ) {
+
+
+	OTF_KeyValuePair pair;
+
+	if ( buffer->list == NULL) {
+
+		return 0;
+	}
+
+	pair.key = OTF_RBuffer_readUint32( buffer );
+
+	if ( OTF_RBuffer_testKeyword( buffer, OTF_KEYWORD_S_LOCAL_TYPE ) ||
+				OTF_RBuffer_testKeyword( buffer, OTF_KEYWORD_L_LOCAL_TYPE ) ) {
+
+		pair.type = (OTF_Type) OTF_RBuffer_readUint32( buffer );
+
+	} else {
+		/* Parse error */
+		PARSE_ERROR( buffer );
+		
+		return 0;
+	}
+	
+	if ( OTF_RBuffer_testKeyword( buffer, OTF_KEYWORD_S_LOCAL_VALUE ) ||
+				OTF_RBuffer_testKeyword( buffer, OTF_KEYWORD_L_LOCAL_VALUE ) ) {
+		switch (pair.type) {
+		case OTF_CHAR:
+			OTF_RBuffer_readBytes( buffer, (uint8_t*) &(pair.value.otf_char), 1 );
+			break;
+		case OTF_INT8:
+		  	pair.value.otf_int8 = OTF_RBuffer_readUint32( buffer );
+			break;
+		case OTF_UINT8:
+			pair.value.otf_uint8 = OTF_RBuffer_readUint32( buffer );
+			break;
+		case OTF_INT16:
+			pair.value.otf_int16 = OTF_RBuffer_readUint32( buffer );
+			break;
+		case OTF_UINT16:
+			pair.value.otf_uint16 = OTF_RBuffer_readUint32( buffer );
+			break;
+		case OTF_INT32:
+			pair.value.otf_int32 = OTF_RBuffer_readUint32( buffer );
+			break;
+		case OTF_UINT32:
+			pair.value.otf_uint32 = OTF_RBuffer_readUint32( buffer );
+			break;
+		case OTF_INT64:
+			pair.value.otf_int64 = OTF_RBuffer_readUint64( buffer );
+			break;
+		case OTF_UINT64:
+			pair.value.otf_uint64 = OTF_RBuffer_readUint64( buffer );
+			break;
+		case OTF_DOUBLE:
+			pair.value.otf_double = OTF_Int64ToDouble( OTF_RBuffer_readUint64( buffer ) );
+			break;
+		case OTF_FLOAT:
+			pair.value.otf_float = OTF_Int32ToFloat( OTF_RBuffer_readUint32( buffer ) );
+			break;
+		case OTF_BYTE_ARRAY:
+			pair.value.otf_byte_array.len = OTF_RBuffer_readBytes( buffer,
+			pair.value.otf_byte_array.array, OTF_KEYVALUE_MAX_ARRAY_LEN);
+
+			if( pair.value.otf_byte_array.len > OTF_KEYVALUE_MAX_ARRAY_LEN ) {
+
+				OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
+						"byte-array exceeds the maximum length of %u bytes per line.\n",
+						__FUNCTION__, __FILE__, __LINE__, OTF_KEYVALUE_MAX_ARRAY_LEN );
+
+				PARSE_ERROR( buffer );
+
+				return 0;
+
+			}
+
+			if ( OTF_RBuffer_testKeyword( buffer, OTF_KEYWORD_S_LOCAL_LENGTH ) ||
+				OTF_RBuffer_testKeyword( buffer, OTF_KEYWORD_L_LOCAL_LENGTH ) ) {
+
+				pair.value.otf_byte_array.len = OTF_RBuffer_readUint32( buffer );
+
+			}
+
+			break;
+		default:
+
+			/* Pasre error */
+			PARSE_ERROR( buffer );
+
+			return 0;
+		}
+
+	} else {
+
+		/* Parse error */
+		PARSE_ERROR( buffer );
+		
+		return 0;
+	}
+
+	if ( OTF_RBuffer_readNewline( buffer ) ) {
+
+		OTF_KeyValueList_appendPair(buffer->list, pair);
+
+	} else {
+
+		PARSE_ERROR( buffer );
+		
+		return 0;
+	}
+
+	return 1;
+}
+
