@@ -1,6 +1,7 @@
 /*
- This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2010.
+ This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2011.
  Authors: Andreas Knuepfer, Holger Brunst, Ronny Brendel, Thomas Kriebitzsch
+ also: patches by Rainer Keller, thanks a lot!
 */
 
 #ifdef HAVE_CONFIG_H
@@ -15,6 +16,8 @@
 #include "OTF_Parse.h"
 #include "OTF_Errno.h"
 
+#include "OTF_Keywords.h"
+
 
 /** constructor - internal use only */
 int OTF_RStream_init( OTF_RStream* rstream );
@@ -69,7 +72,7 @@ int OTF_RStream_finish( OTF_RStream* rstream ) {
 		ret &= tmpret;
 		if( 0 == tmpret ) {
 			
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"cannot close defbuffer.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 		}
@@ -82,7 +85,7 @@ int OTF_RStream_finish( OTF_RStream* rstream ) {
 		ret &= tmpret;
 		if( 0 == tmpret ) {
 			
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"cannot close event buffer.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 	
@@ -96,7 +99,7 @@ int OTF_RStream_finish( OTF_RStream* rstream ) {
 		ret &= tmpret;
 		if( 0 == tmpret ) {
 			
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"cannot close snapshots buffer.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 
@@ -110,7 +113,7 @@ int OTF_RStream_finish( OTF_RStream* rstream ) {
 		ret &= tmpret;
 		if( 0 == tmpret ) {
 			
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"cannot close statistics buffer.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 
@@ -124,7 +127,7 @@ int OTF_RStream_finish( OTF_RStream* rstream ) {
 		ret &= tmpret;
 		if( 0 == tmpret ) {
 			
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"cannot close defbuffer.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 
@@ -139,10 +142,22 @@ int OTF_RStream_finish( OTF_RStream* rstream ) {
 OTF_RStream* OTF_RStream_open( const char* namestub, uint32_t id, OTF_FileManager* manager ) {
 
 
-	OTF_RStream* ret= (OTF_RStream*) malloc( sizeof(OTF_RStream) );
+	OTF_RStream* ret;
+
+	/* Check the input parameters */
+	if( NULL == manager ) {
+		
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
+				"manager has not been specified.\n",
+				__FUNCTION__, __FILE__, __LINE__ );
+
+		return NULL;
+	}
+
+	ret= (OTF_RStream*) malloc( sizeof(OTF_RStream) );
 	if( NULL == ret ) {
 		
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"no memory left.\n",
 				__FUNCTION__, __FILE__, __LINE__ );
 
@@ -154,17 +169,6 @@ OTF_RStream* OTF_RStream_open( const char* namestub, uint32_t id, OTF_FileManage
 	ret->namestub= strdup( namestub );
 	ret->id= id;
 
-	if( NULL == manager ) {
-		
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
-				"manager has not been specified.\n",
-				__FUNCTION__, __FILE__, __LINE__ );
-
-		free( ret );
-		ret= NULL;
-
-		return NULL;
-	}
 	ret->manager= manager;
 
 	/* leave buffers alone, they are allocated on demand */
@@ -180,7 +184,7 @@ int OTF_RStream_close( OTF_RStream* rstream ) {
 	
 	if( NULL == rstream ) {
 		
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"rstream has not been specified.\n",
 				__FUNCTION__, __FILE__, __LINE__ );
 
@@ -208,7 +212,7 @@ OTF_RBuffer* OTF_RStream_getDefBuffer( OTF_RStream* rstream ) {
 			rstream->id, OTF_FILETYPE_DEF, 0, NULL );
 		if( NULL == filename ) {
 			
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"OTF_getFilename() failed.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 
@@ -234,6 +238,24 @@ OTF_RBuffer* OTF_RStream_getDefBuffer( OTF_RStream* rstream ) {
 }
 
 
+OTF_RBuffer* OTF_RStream_setDefBuffer( OTF_RStream* rstream, OTF_RBuffer* rbuffer ) {
+
+
+    OTF_RBuffer* old= rstream->defBuffer;
+    rstream->defBuffer= rbuffer;
+
+    /* the following is important because otherwise the buffer sizes stay 0 
+    which leads to weird problems down the road */
+
+    OTF_RBuffer_setSize( rstream->defBuffer, rstream->buffersizes );
+#ifdef HAVE_ZLIB
+    OTF_RBuffer_setZBufferSize( rstream->defBuffer, rstream->zbuffersizes );
+#endif /* HAVE_ZLIB */
+
+    return old;
+}
+
+
 int OTF_RStream_closeDefBuffer( OTF_RStream* rstream ) {
 
 
@@ -244,7 +266,7 @@ int OTF_RStream_closeDefBuffer( OTF_RStream* rstream ) {
 		ret&= OTF_RBuffer_close( rstream->defBuffer );
 		if( 0 == ret ) {
 			
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"closing defbuffer failed.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 
@@ -269,7 +291,7 @@ OTF_RBuffer* OTF_RStream_getEventBuffer( OTF_RStream* rstream ) {
 			rstream->id, OTF_FILETYPE_EVENT, 0, NULL );
 		if( NULL == filename ) {
 			
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"OTF_getFilename() failed for event file with name stub '%s' and ID %u.\n",
 					__FUNCTION__, __FILE__, __LINE__, rstream->namestub, rstream->id );
 
@@ -277,20 +299,19 @@ OTF_RBuffer* OTF_RStream_getEventBuffer( OTF_RStream* rstream ) {
 		}
 
 		rstream->eventBuffer= OTF_RBuffer_open( filename, rstream->manager );
-		if ( NULL == rstream->eventBuffer ) {
-
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
-					"OTF_RBuffer_open() failed for filename '%s'.\n",
-					__FUNCTION__, __FILE__, __LINE__, filename );
-
-			free( filename );
-			filename = NULL;
-			return NULL;
-		}
-
 		free( filename );
 		filename = NULL;
 
+		if ( NULL == rstream->eventBuffer ) {
+
+/* *** commented because it can happen when file not exists
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
+					"OTF_RBuffer_open() failed for filename '%s'.\n",
+					__FUNCTION__, __FILE__, __LINE__, filename );
+*/
+
+			return NULL;
+		}
 
 		OTF_RBuffer_setSize( rstream->eventBuffer, rstream->buffersizes );
 #ifdef HAVE_ZLIB
@@ -312,7 +333,7 @@ int OTF_RStream_closeEventBuffer( OTF_RStream* rstream ) {
 		ret&= OTF_RBuffer_close( rstream->eventBuffer );
 		if( 0 == ret ) {
 			
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"closing event buffer failed.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 
@@ -336,7 +357,7 @@ OTF_RBuffer* OTF_RStream_getSnapsBuffer( OTF_RStream* rstream ) {
 			rstream->id, OTF_FILETYPE_SNAPS, 0, NULL );
 		if( NULL == filename ) {
 			
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"OTF_getFilename() failed.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 
@@ -372,7 +393,7 @@ int OTF_RStream_closeSnapsBuffer( OTF_RStream* rstream ) {
 		ret&= OTF_RBuffer_close( rstream->snapsBuffer );
 		if( 0 == ret ) {
 			
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"closing snapshots buffer failed.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 
@@ -396,7 +417,7 @@ OTF_RBuffer* OTF_RStream_getStatsBuffer( OTF_RStream* rstream ) {
 			rstream->id, OTF_FILETYPE_STATS, 0, NULL );
 		if( NULL == filename ) {
 			
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"OTF_getFilename() failed.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 
@@ -410,10 +431,6 @@ OTF_RBuffer* OTF_RStream_getStatsBuffer( OTF_RStream* rstream ) {
 
 		if ( NULL == rstream->statsBuffer ) {
 
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
-					"OTF_RBuffer_open() failed.\n",
-					__FUNCTION__, __FILE__, __LINE__ );
-
 			return NULL;
 		}
 
@@ -437,7 +454,7 @@ int OTF_RStream_closeStatsBuffer( OTF_RStream* rstream ) {
 		ret&= OTF_RBuffer_close( rstream->statsBuffer );
 		if( 0 == ret ) {
 			
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"closing statistics buffer failed.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 
@@ -461,15 +478,13 @@ OTF_RBuffer* OTF_RStream_getMarkerBuffer( OTF_RStream* rstream ) {
 			rstream->id, OTF_FILETYPE_MARKER, 0, NULL );
 		if( NULL == filename ) {
 			
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"OTF_getFilename() failed.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 
 			return NULL;
 		}
 
-		OTF_fprintf( stderr, "opening file '%s'.\n", filename );
-
 		rstream->markerBuffer= OTF_RBuffer_open( filename, rstream->manager );
 		free( filename );
 		filename = NULL;
@@ -499,7 +514,7 @@ int OTF_RStream_closeMarkerBuffer( OTF_RStream* rstream ) {
 		ret&= OTF_RBuffer_close( rstream->markerBuffer );
 		if( 0 == ret ) {
 			
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"closing defbuffer failed.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 
@@ -517,7 +532,7 @@ void OTF_RStream_setBufferSizes( OTF_RStream* rstream, uint32_t size ) {
 
 	if ( 50 > size ) {
 	
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"intended buffer size %u is too small, rejected.\n",
 				__FUNCTION__, __FILE__, __LINE__, size );
 		
@@ -525,13 +540,13 @@ void OTF_RStream_setBufferSizes( OTF_RStream* rstream, uint32_t size ) {
 
 	} else if ( 500 > size ) {
 	
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Warning( "WARNING in function %s, file: %s, line: %i:\n "
 				"buffer size %u is very small, accepted though.\n",
 				__FUNCTION__, __FILE__, __LINE__, size );
 
 	} else if ( 10 * 1024 *1024 < size ) {
 
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Warning( "WARNING in function %s, file: %s, line: %i:\n "
 				"buffer size %u is rather big, accepted though.\n",
 				__FUNCTION__, __FILE__, __LINE__, size );
 
@@ -555,7 +570,7 @@ void OTF_RStream_setZBufferSizes( OTF_RStream* rstream, uint32_t size ) {
 	
 	if ( 32 > size ) {
 	
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"intended zbuffer size %u is too small, rejected.\n",
 				__FUNCTION__, __FILE__, __LINE__, size );
 		
@@ -563,13 +578,13 @@ void OTF_RStream_setZBufferSizes( OTF_RStream* rstream, uint32_t size ) {
 
 	} else if ( 512 > size ) {
 	
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Warning( "WARNING in function %s, file: %s, line: %i:\n "
 				"zbuffer size %u is very small, accepted though.\n",
 				__FUNCTION__, __FILE__, __LINE__, size );
 
 	} else if ( 10 * 1024 *1024 < size ) {
 
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Warning( "WARNING in function %s, file: %s, line: %i:\n "
 				"zbuffer size %u is rather big, accepted though.\n",
 				__FUNCTION__, __FILE__, __LINE__, size );
 
@@ -597,7 +612,7 @@ void OTF_RStream_setRecordLimit( OTF_RStream* rstream, uint64_t limit ) {
 
 	if( limit == OTF_READ_ERROR ) {
 
-		OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"limit cannot be set to %llu. Reset to OTF_READ_MAXRECORDS.\n",
 				__FUNCTION__, __FILE__, __LINE__,
 				(long long unsigned) limit );
@@ -624,6 +639,7 @@ uint64_t OTF_RStream_readDefinitions( OTF_RStream* rstream, OTF_HandlerArray* ha
 
 	int ret;
 
+	char next_char = '\0';
 
 	/* initialized? */
 	if ( NULL == rstream->defBuffer ) {
@@ -635,7 +651,7 @@ uint64_t OTF_RStream_readDefinitions( OTF_RStream* rstream, OTF_HandlerArray* ha
 
 		if ( NULL == rstream->defBuffer ) {
 		
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"the stream has no def buffer.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 			
@@ -653,14 +669,24 @@ uint64_t OTF_RStream_readDefinitions( OTF_RStream* rstream, OTF_HandlerArray* ha
 			return recordcount;
 		}
 
+		/* remember next record type, if it will be a none
+		   KEYVALUE record, dont't account it in recordcount */
+		next_char = *(rstream->defBuffer->buffer + rstream->defBuffer->pos);
+
 		ret= OTF_Reader_parseDefRecord( rstream->defBuffer, handlers, rstream->id );
 		if ( 0 == ret ) {
 
 			/* maybe later an errorhandler gives the record to the user */
 			return OTF_READ_ERROR;
 		}
-
-		recordcount++;
+	
+		/* Now reset the KeyValue list, if we consumed a none
+		   KEYVALUE record */
+		if ( next_char != OTF_KEYWORD_F_KEYVALUE_PREFIX /* 'K' */ ) {
+			OTF_KeyValueList_reset(rstream->defBuffer->list);
+			recordcount++;
+		}
+		
 	}
 
 	return recordcount;
@@ -679,6 +705,7 @@ uint64_t OTF_RStream_readEvents( OTF_RStream* rstream, OTF_HandlerArray* handler
 		uint64_t oldtime= 0;
 #	endif
 
+	char next_char = '\0';
 
 	/* initialized? */
 	if ( NULL == rstream->eventBuffer ) {
@@ -689,7 +716,7 @@ uint64_t OTF_RStream_readEvents( OTF_RStream* rstream, OTF_HandlerArray* handler
 		rstream->eventBuffer= OTF_RStream_getEventBuffer( rstream );
 		if( NULL == rstream->eventBuffer ) {
 			
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"the stream has no event buffer.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 			
@@ -718,7 +745,7 @@ uint64_t OTF_RStream_readEvents( OTF_RStream* rstream, OTF_HandlerArray* handler
 			
 			if ( oldtime > currenttime ) {
 		
-				OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+				OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"Time does decrease!!! %llu < %llu.\n",
 					__FUNCTION__, __FILE__, __LINE__,
 					(unsigned long long) currenttime,
@@ -728,6 +755,10 @@ uint64_t OTF_RStream_readEvents( OTF_RStream* rstream, OTF_HandlerArray* handler
 			}
 #		endif
 
+		/* remember next record type, if it will be a none
+		   KEYVALUE record, dont't account it in recordcount */
+		next_char = *(rstream->eventBuffer->buffer + rstream->eventBuffer->pos);
+
 		ret= OTF_Reader_parseEventRecord( rstream->eventBuffer, handlers );
 		if ( 0 == ret ) {
 
@@ -735,7 +766,13 @@ uint64_t OTF_RStream_readEvents( OTF_RStream* rstream, OTF_HandlerArray* handler
 			return OTF_READ_ERROR;
 		}
 
-		recordcount++;
+		/* Now reset the KeyValue list, if we consumed a none
+		   KEYVALUE record */
+		if ( next_char != OTF_KEYWORD_F_KEYVALUE_PREFIX /* 'K' */ ) {
+			OTF_KeyValueList_reset(rstream->eventBuffer->list);
+			recordcount++;
+		}
+		
 	}
 
 	return recordcount;
@@ -754,6 +791,7 @@ uint64_t OTF_RStream_readSnapshots( OTF_RStream* rstream, OTF_HandlerArray* hand
 		uint64_t oldtime= 0;
 #	endif
 
+	char next_char = '\0';
 
 	/* initialized? */
 	if ( NULL == rstream->snapsBuffer ) {
@@ -764,7 +802,7 @@ uint64_t OTF_RStream_readSnapshots( OTF_RStream* rstream, OTF_HandlerArray* hand
 		rstream->snapsBuffer= OTF_RStream_getSnapsBuffer( rstream );
 		if( NULL == rstream->snapsBuffer ) {
 			
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"the stream has no snapshots buffer.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 			
@@ -793,7 +831,7 @@ uint64_t OTF_RStream_readSnapshots( OTF_RStream* rstream, OTF_HandlerArray* hand
 			
 			if ( oldtime > currenttime ) {
 		
-				OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+				OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"Time does decrease!!! %llu < %llu.\n",
 					__FUNCTION__, __FILE__, __LINE__,
 					(unsigned long long) currenttime,
@@ -803,6 +841,9 @@ uint64_t OTF_RStream_readSnapshots( OTF_RStream* rstream, OTF_HandlerArray* hand
 			}
 #		endif
 
+		/* remember next record type, if it will be a none
+		   KEYVALUE record, dont't account it in recordcount */
+		next_char = *(rstream->snapsBuffer->buffer + rstream->snapsBuffer->pos);
 
 		ret= OTF_Reader_parseSnapshotsRecord( rstream->snapsBuffer, handlers );
 		if ( 0 == ret ) {
@@ -810,11 +851,16 @@ uint64_t OTF_RStream_readSnapshots( OTF_RStream* rstream, OTF_HandlerArray* hand
 			/* maybe later an errorhandler gives the record to the user */
 			return OTF_READ_ERROR;
 		}
+		
+		/* Now reset the KeyValue list, if we consumed a none
+		   KEYVALUE record */
+		if ( next_char != OTF_KEYWORD_F_KEYVALUE_PREFIX /* 'K' */ ) {
+			OTF_KeyValueList_reset(rstream->snapsBuffer->list);
+			recordcount++;
+		}
 
-		recordcount++;
 	}
 
-
 	return recordcount;
 }
 
@@ -831,6 +877,7 @@ uint64_t OTF_RStream_readStatistics( OTF_RStream* rstream, OTF_HandlerArray* han
 		uint64_t oldtime= 0;
 #	endif
 
+	char next_char = '\0';
 
 	/* initialized? */
 	if ( NULL == rstream->statsBuffer ) {
@@ -841,7 +888,7 @@ uint64_t OTF_RStream_readStatistics( OTF_RStream* rstream, OTF_HandlerArray* han
 		rstream->statsBuffer= OTF_RStream_getStatsBuffer( rstream );
 		if( NULL == rstream->statsBuffer ) {
 			
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"the stream has no statistics buffer.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 			
@@ -870,7 +917,7 @@ uint64_t OTF_RStream_readStatistics( OTF_RStream* rstream, OTF_HandlerArray* han
 
 			if ( oldtime > currenttime ) {
 		
-				OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+				OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"Time does decrease!!! %llu < %llu.\n",
 					__FUNCTION__, __FILE__, __LINE__,
 					(unsigned long long) currenttime,
@@ -880,6 +927,9 @@ uint64_t OTF_RStream_readStatistics( OTF_RStream* rstream, OTF_HandlerArray* han
 			}
 #		endif
 
+		/* remember next record type, if it will be a none
+		   KEYVALUE record, dont't account it in recordcount */
+		next_char = *(rstream->statsBuffer->buffer + rstream->statsBuffer->pos);
 
 		ret= OTF_Reader_parseStatisticsRecord( rstream->statsBuffer, handlers );
 		if ( 0 == ret ) {
@@ -888,11 +938,17 @@ uint64_t OTF_RStream_readStatistics( OTF_RStream* rstream, OTF_HandlerArray* han
 			/* maybe later an errorhandler gives the record to the user */
 			return OTF_READ_ERROR;
 		}
+		
+		/* Now reset the KeyValue list, if we consumed a none
+		   KEYVALUE record */
+		if ( next_char != OTF_KEYWORD_F_KEYVALUE_PREFIX /* 'K' */ ) {
+			OTF_KeyValueList_reset(rstream->statsBuffer->list);
+			recordcount++;
+		}
+
 
-		recordcount++;
 	}
 
-
 	return recordcount;
 }
 
@@ -904,6 +960,7 @@ uint64_t OTF_RStream_readMarker( OTF_RStream* rstream, OTF_HandlerArray* handler
 
 	int ret;
 
+	char next_char = '\0';
 
 	/* initialized? */
 	if ( NULL == rstream->markerBuffer ) {
@@ -915,7 +972,7 @@ uint64_t OTF_RStream_readMarker( OTF_RStream* rstream, OTF_HandlerArray* handler
 
 		if ( NULL == rstream->markerBuffer ) {
 		
-			OTF_fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"the stream has no def buffer.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 			
@@ -933,6 +990,10 @@ uint64_t OTF_RStream_readMarker( OTF_RStream* rstream, OTF_HandlerArray* handler
 			return recordcount;
 		}
 
+		/* remember next record type, if it will be a none
+		   KEYVALUE record, dont't account it in recordcount */
+		next_char = *(rstream->markerBuffer->buffer + rstream->markerBuffer->pos);
+
 		ret= OTF_Reader_parseMarkerRecord( rstream->markerBuffer, handlers, rstream->id );
 		if ( 0 == ret ) {
 
@@ -940,7 +1001,13 @@ uint64_t OTF_RStream_readMarker( OTF_RStream* rstream, OTF_HandlerArray* handler
 			return OTF_READ_ERROR;
 		}
 
-		recordcount++;
+		/* Now reset the KeyValue list, if we consumed a none
+		   KEYVALUE record */
+		if ( next_char != OTF_KEYWORD_F_KEYVALUE_PREFIX /* 'K' */ ) {
+			OTF_KeyValueList_reset(rstream->markerBuffer->list);
+			recordcount++;
+		}
+
 	}
 
 	return recordcount;
