@@ -1,12 +1,15 @@
 /*
- This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2008.
+ This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2011.
  Authors: Andreas Knuepfer, Holger Brunst, Ronny Brendel, Thomas Kriebitzsch
 */
 
 #ifndef HASH_H
 #define HASH_H
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+
 
 #include "OTF_inttypes.h"
 
