@@ -206,7 +206,7 @@ namespace {
     pragma->find_name();
     pragma->pline=save_pline;               // reset parse position
     pragma->ppos=save_ppos;
-    if(pragma->name.find("do")!=string::npos) {
+    if(pragma->name.find("do ")!=string::npos) {
       linetype=PRAGMA_LOOPSTART;
       if(pragma->name == "enddo")              linetype=PRAGMA_LOOPEND;
       else if(pragma->name == "paralleldo")    linetype=PRAGMA_PARLOOPSTART;
@@ -417,6 +417,15 @@ void process_fortran(istream& is, const char* infile, ostream& os,
         currPragma->lines.push_back(lowline);
       } else {
         // new directive
+        if ( currPragma ) {
+          // if necessary process last complete directive
+          typeOfLastLine = check_pragma(currPragma);
+          test_and_insert_ompenddo(os, typeOfLastLine, waitforOMPEndDo,
+                                   infile, currPragma->lineno, pragma_indent,
+                                   pomp, addSharedDecl);
+          process_pragma(currPragma, os);
+          currPragma = 0;
+        }
         currPragma
                 = new OMPragmaF(infile, lineno, pstart+5+pomp, lowline, pomp,
                                 addSharedDecl);
@@ -424,7 +433,7 @@ void process_fortran(istream& is, const char* infile, ostream& os,
       string::size_type com = lowline.find("!", pstart+4+pomp);
       if ( com != string::npos ) --com;
       string::size_type amp = lowline.find_last_not_of(" \t", com);
-      if ( lowline[amp] == '&' ) {
+      if ( amp != string::npos && lowline[amp] == '&' ) {
         // last non-white non-comment character == '&' --> continuation
         needPragma = true;
       } else {
@@ -532,4 +541,9 @@ void process_fortran(istream& is, const char* infile, ostream& os,
       }
     }
   }
+
+  // currPragma should be deleted at this point; ensure that to prevent
+  // memory leak
+  if ( currPragma )
+    delete currPragma;
 }
