@@ -2,7 +2,7 @@
  * VampirTrace
  * http://www.tu-dresden.de/zih/vampirtrace
  *
- * Copyright (c) 2005-2010, ZIH, TU Dresden, Federal Republic of Germany
+ * Copyright (c) 2005-2011, ZIH, TU Dresden, Federal Republic of Germany
  *
  * Copyright (c) 1998-2005, Forschungszentrum Juelich, Juelich Supercomputing
  *                          Centre, Federal Republic of Germany
@@ -27,6 +27,7 @@
 #include <unistd.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <sys/wait.h>
 
 static bool getParams( int argc, char** argv );
 static void showUsage( void );
@@ -160,6 +161,14 @@ getParams( int argc, char** argv )
       showUsage();
       exit(0);
     }
+    // -V, --version
+    //
+    if( strcmp( argv[i], "-V" ) == 0 ||
+        strcmp( argv[i], "--version" ) == 0 )
+    {
+      std::cout << VT_LIBWRAPGEN_VERSION << std::endl;
+      exit(0);
+    }
     // -q, --quiet
     //
     else if( strcmp( argv[i], "-q" ) == 0 ||
@@ -571,14 +580,16 @@ showUsage()
     << "     --build             Build a wrapper library from a generated source file." << std::endl
     << "                         See 'build-options' below for valid options." << std::endl
     << std::endl
+    << "     -h, --help          Show this help message." << std::endl
+    << std::endl
+    << "     -V, --version       Show VampirTrace version." << std::endl
+    << std::endl
     << "     -q, --quiet         Enable quiet mode." << std::endl
     << "                         (only emergency output)" << std::endl
     << std::endl
     << "     -v, --verbose       Increase output verbosity." << std::endl
     << "                         (can be used more than once)" << std::endl
     << std::endl
-    << "     -h, --help          Show this help message." << std::endl
-    << std::endl
     << "   gen-options:" << std::endl
     << "     -o, --output=FILE   Pathname of output wrapper source file." << std::endl
     << "                         (default: "VT_LIBWRAPGEN_DEFAULT_OUTPUT_SRC_FILE")" << std::endl
@@ -923,8 +934,7 @@ GeneratorC::writeHead()
   ts[strlen(ts)-1] = '\0';
 
   out << "/**" << std::endl
-      << " * Generated by '" << ExeName << "' which is a part of "
-      << PACKAGE_STRING << std::endl
+      << " * Generated by '" << ExeName << "' which is a part of VampirTrace" << std::endl
       << " * on " << ts << "." << std::endl
       << " *" << std::endl
       << " * command:" << std::endl
