@@ -9,7 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007-2009 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2006-2007 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2006-2007 Voltaire All rights reserved.
@@ -26,7 +26,7 @@
 
 #include "opal/class/opal_list.h"
 #include "opal/event/event.h"
-#include "ompi/mca/pml/pml.h"
+#include "opal/util/output.h"
 #include "ompi/mca/btl/btl.h"
 #include "btl_openib.h"
 #include "btl_openib_frag.h"
@@ -94,6 +94,12 @@ typedef struct mca_btl_openib_rem_info_t {
     mca_btl_openib_rem_qp_info_t *rem_qps;
     /* Remote xrc_srq info, used only with XRC connections */
     mca_btl_openib_rem_srq_info_t *rem_srqs;
+    /* Vendor id of remote HCA */
+    uint32_t rem_vendor_id;
+    /* Vendor part id of remote HCA */
+    uint32_t rem_vendor_part_id;
+    /* Transport type of remote port */
+    mca_btl_openib_transport_type_t rem_transport_type;
 } mca_btl_openib_rem_info_t;
 
 
@@ -493,8 +499,10 @@ static inline int post_send(mca_btl_openib_endpoint_t *ep,
         sr_desc->opcode = IBV_WR_RDMA_WRITE;
         MCA_BTL_OPENIB_RDMA_FRAG_SET_SIZE(ftr, sg->length);
         MCA_BTL_OPENIB_RDMA_MAKE_LOCAL(ftr);
-#if OMPI_ENABLE_DEBUG
-        ftr->seq = ep->eager_rdma_remote.seq++;
+#if OPAL_ENABLE_DEBUG
+	do {
+	  ftr->seq = ep->eager_rdma_remote.seq;
+	} while (!OPAL_ATOMIC_CMPSET_32((int32_t*)&ep->eager_rdma_remote.seq, (int32_t)ftr->seq, (int32_t)ftr->seq+1));
 #endif
         if(ep->nbo)
             BTL_OPENIB_FOOTER_HTON(*ftr);
@@ -513,7 +521,7 @@ static inline int post_send(mca_btl_openib_endpoint_t *ep,
             sr_desc->opcode = IBV_WR_SEND;
         } else {
             sr_desc->opcode = IBV_WR_SEND_WITH_IMM;
-#if !defined(WORDS_BIGENDIAN) && OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+#if !defined(WORDS_BIGENDIAN) && OPAL_ENABLE_HETEROGENEOUS_SUPPORT
             sr_desc->imm_data = htonl(ep->rem_info.rem_index);
 #else
             sr_desc->imm_data = ep->rem_info.rem_index;
