@@ -71,6 +71,7 @@
 
 #include "opal/mca/maffinity/base/base.h"
 #include "opal/mca/paffinity/base/base.h"
+#include "opal/mca/hwloc/base/base.h"
 #include "opal/class/opal_pointer_array.h"
 #include "opal/util/opal_environ.h"
 
@@ -130,12 +131,12 @@ static bool odls_default_child_died(pid_t pid, unsigned int timeout, int *exit_s
 {
     time_t end;
     pid_t ret;
-#if !defined(HAVE_SCHED_YIELD)
-    struct timeval t;
-    fd_set bogus;
-#endif
         
-    end = time(NULL) + timeout;
+    /* Because of rounding in time (which returns whole seconds) we
+     * have to add 1 to our wait number: this means that we wait
+     * somewhere between (target) and (target)+1 seconds.  Otherwise,
+     * the default 1s actually means 'somwhere between 0 and 1s'. */
+    end = time(NULL) + timeout + 1;
     do {
         ret = waitpid(pid, exit_status, WNOHANG);
         if (pid == ret) {
@@ -150,13 +151,17 @@ static bool odls_default_child_died(pid_t pid, unsigned int timeout, int *exit_s
              * as there is no error - this is a race condition problem
              * that occasionally causes us to incorrectly report a proc
              * as refusing to die. Unfortunately, errno may not be reset
-             * by waitpid in this case, so we cannot check it - just assume
-             * the proc has indeed died
+             * by waitpid in this case, so we cannot check it.
+	     *
+	     * (note the previous fix to this, to return 'process dead'
+	     * here, fixes the race condition at the cost of reporting
+	     * all live processes have immediately died!  Better to
+	     * occasionally report a dead process as still living -
+	     * which will occasionally trip the timeout for cases that
+	     * are right on the edge.)
              */
-            OPAL_OUTPUT_VERBOSE((2, orte_odls_globals.output,
-                                 "%s odls:default:WAITPID INDICATES PROC %d HAS ALREADY EXITED",
-                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), (int)pid));
-            return true;
+
+	    /* Do nothing, process still alive */
         } else if (-1 == ret && ECHILD == errno) {
             /* The pid no longer exists, so we'll call this "good
                enough for government work" */
@@ -166,17 +171,14 @@ static bool odls_default_child_died(pid_t pid, unsigned int timeout, int *exit_s
             return true;
         }
         
-#if defined(HAVE_SCHED_YIELD)
-        sched_yield();
-#else
-        /* Bogus delay for 1 usec */
-        t.tv_sec = 0;
-        t.tv_usec = 1;
-        FD_ZERO(&bogus);
-        FD_SET(0, &bogus);
-        select(1, &bogus, NULL, NULL, &t);
-#endif
-        
+        /* Bogus delay for 1 msec - let's actually give the CPU some time
+         * to quit the other process (sched_yield() -- even if we have it
+         * -- changed behavior in 2.6.3x Linux flavors to be undesirable)
+         * Don't use select on a bogus file descriptor here as it has proven
+         * unreliable and sometimes immediately returns - we really, really
+         * -do- want to wait a bit!
+         */
+        usleep(1000);
     } while (time(NULL) < end);
 
     /* The child didn't die, so return false */
@@ -383,7 +385,7 @@ static int odls_default_fork_local_proc(orte_app_context_t* context,
                 OPAL_PAFFINITY_CPU_ZERO(mask);
                 if (ORTE_MAPPING_NPERXXX & jobdat->policy) {
                     /* we need to balance the children from this job across the available sockets */
-                    npersocket = jobdat->num_local_procs / orte_odls_globals.num_sockets;
+                    npersocket = jobdat->npersocket;
                     /* determine the socket to use based on those available */
                     if (npersocket < 2) {
                         /* if we only have 1/sock, or we have less procs than sockets,
@@ -578,7 +580,7 @@ static int odls_default_fork_local_proc(orte_app_context_t* context,
                 }
                 if (ORTE_MAPPING_NPERXXX & jobdat->policy) {
                     /* we need to balance the children from this job across the available sockets */
-                    npersocket = jobdat->num_local_procs / orte_odls_globals.num_sockets;
+                    npersocket = jobdat->npersocket;
                     /* determine the socket to use based on those available */
                     if (npersocket < 2) {
                         /* if we only have 1/sock, or we have less procs than sockets,
@@ -702,7 +704,7 @@ static int odls_default_fork_local_proc(orte_app_context_t* context,
                             }
                         } else {
                             /* compute the logical socket, compensating for the number of cpus_per_rank */
-                            logical_skt = lrank / (orte_default_num_cores_per_socket / jobdat->cpus_per_rank);
+                            logical_skt = lrank / (orte_odls_globals.num_cores_per_socket / jobdat->cpus_per_rank);
                             /* wrap that around the number of sockets so we round-robin */
                             logical_skt = logical_skt % orte_odls_globals.num_sockets;
                             /* now get the target physical socket */
@@ -724,7 +726,7 @@ static int odls_default_fork_local_proc(orte_app_context_t* context,
                 
                 OPAL_PAFFINITY_CPU_ZERO(mask);
                 
-                for (n=0; n < orte_default_num_cores_per_socket; n++) {
+                for (n=0; n < orte_odls_globals.num_cores_per_socket; n++) {
                     /* get the physical core within this target socket */
                     phys_core = opal_paffinity_base_get_physical_core_id(target_socket, n);
                     if (0 > phys_core) {
@@ -758,7 +760,8 @@ static int odls_default_fork_local_proc(orte_app_context_t* context,
                 OPAL_PAFFINITY_PROCESS_IS_BOUND(mask, &bound);
                 if (!bound) {
                     orte_show_help("help-odls-default.txt",
-                                   "odls-default:could-not-bind-to-socket", true);
+                                   "odls-default:could-not-bind-to-socket", true,
+                                   target_socket, orte_process_info.nodename);
                     ORTE_ODLS_ERROR_OUT(ORTE_ERR_FATAL);
                 }
                 if (orte_report_bindings) {
@@ -815,6 +818,9 @@ LAUNCH_PROCS:
                 opal_setenv(param, tmp, true, &environ_copy);
                 free(tmp);
             }
+
+            /* Also set the memory affininty policy */
+            opal_hwloc_base_set_process_membind_policy();
         }
 
         /* close all file descriptors w/ exception of
