@@ -11,6 +11,8 @@
  *                         All rights reserved.
  * Copyright (c) 2007-2011 Oracle and/or its affiliates.  All rights reserved.
  * Copyright (c) 2007      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c)      2011 Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -67,7 +69,7 @@ ORTE_DECLSPEC extern orte_process_name_t orte_name_invalid;  /** instantiated in
 /* define the name of my daemon */
 #define ORTE_PROC_MY_DAEMON     (&orte_process_info.my_daemon)
 
-/* See comment in orte/tools/orterun/debuggers.c about this MCA
+/* See comment in orte/tools/orterun.c about this MCA
    param */
 ORTE_DECLSPEC extern bool orte_in_parallel_debugger;
 
@@ -167,7 +169,7 @@ typedef struct {
     /** Parent object */
     opal_object_t super;
     /** Unique index when multiple apps per job */
-    int8_t idx;
+    int32_t idx;
     /** Absolute pathname of argv[0] */
     char   *app;
     /** Number of copies of this process that are to be launched */
@@ -341,7 +343,7 @@ typedef struct {
     /* app_context array for this job */
     opal_pointer_array_t *apps;
     /* number of app_contexts in the array */
-    orte_std_cntr_t num_apps;
+    int32_t num_apps;
     /* flags to control the launch of this job - see above
      * for description of supported flags
      */
@@ -423,7 +425,7 @@ struct orte_proc_t {
     /* exit code */
     orte_exit_code_t exit_code;
     /* the app_context that generated this proc */
-    int8_t app_idx;
+    int32_t app_idx;
     /* a cpu list, if specified by the user */
     char *slot_list;
     /* pointer to the node where this proc is executing */
@@ -557,8 +559,12 @@ ORTE_DECLSPEC extern bool orte_allocation_required;
 ORTE_DECLSPEC extern char *orte_launch_agent;
 ORTE_DECLSPEC extern char **orted_cmd_line;
 
+/* debugger job */
 ORTE_DECLSPEC extern orte_job_t *orte_debugger_daemon;
-ORTE_DECLSPEC extern bool orte_enable_debug_cospawn_while_running;
+ORTE_DECLSPEC extern bool orte_debugger_dump_proctable;
+ORTE_DECLSPEC extern char *orte_debugger_test_daemon;
+ORTE_DECLSPEC extern bool orte_debugger_test_attach;
+ORTE_DECLSPEC extern bool orte_debugger_enable_fifo_attach;
 ORTE_DECLSPEC extern int orte_debugger_check_rate;
 
 /* exit triggers and flags */
